@@ -1,23 +1,24 @@
 /* JitterLisp: s-expression header.
 
    Copyright (C) 2017, 2018, 2019, 2020 Luca Saiu
+   Updated in 2021 by Luca Saiu
    Written by Luca Saiu
 
    This file is part of the JitterLisp language implementation, distributed as
-   an example along with Jitter under the same license.
+   an example along with GNU Jitter under the same license.
 
-   Jitter is free software: you can redistribute it and/or modify
+   GNU Jitter is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   Jitter is distributed in the hope that it will be useful,
+   GNU Jitter is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with Jitter.  If not, see <http://www.gnu.org/licenses/>. */
+   along with GNU Jitter.  If not, see <http://www.gnu.org/licenses/>. */
 
 
 #ifndef JITTERLISP_SEXPRESSION_H_
@@ -285,7 +286,7 @@ jitterlisp_unique_object_names [];
 
 /* How many unique objects there are.  This of course must agree with the
    definitions below. */
-#define JITTERLISP_UNIQUE_OBJECT_NO  6
+#define JITTERLISP_UNIQUE_OBJECT_NO  10
 
 /* Define every unique object.  The names in the definition of
    jitterlisp_unique_object_names must follow this order. */
@@ -314,6 +315,13 @@ jitterlisp_unique_object_names [];
    a correct evaluation.  No reader syntax. */
 #define JITTERLISP_UNDEFINED   JITTERLISP_UNIQUE_ENCODE(5)
 
+/* Other unique objects which should never be visible to the user as the result
+   of any evaluation, but are useful internally for debugging purposes. */
+#define JITTERLISP_UNINITIALIZED_REGISTER_VALUE     JITTERLISP_UNIQUE_ENCODE(6)
+#define JITTERLISP_UNINITIALIZED_MAINSTACK_VALUE    JITTERLISP_UNIQUE_ENCODE(7)
+#define JITTERLISP_UNINITIALIZED_RETURNSTACK_VALUE  JITTERLISP_UNIQUE_ENCODE(8)
+#define JITTERLISP_UNINITIALIZED_HEAP_VALUE         JITTERLISP_UNIQUE_ENCODE(9)
+
 /* Unique object predicates.  Since unique objects are unboxed (and unique)
    these are simple comparisons by identity.  The expansion evaluates to a
    C (untagged) boolean r-value. */
@@ -560,7 +568,7 @@ struct jitterlisp_compiled_closure
      time.
      This is actually a jitterlispvm_program_point object, but we declare it as
      a const void * (which is safe: jitterlispvm_program_point is a constant
-     pointer type on every dispatching model) to avoid cyclical CPP inclusion
+     pointer type on every dispatch) to avoid cyclical CPP inclusion
      problems. */
   const void *first_program_point;
 };
@@ -651,8 +659,11 @@ struct jitterlisp_primitive
 {
   /* Make the first field, and therefore the whole struct, double-word aligned:
      this ensures that enough low-order zero bits in the address can be
-     overwritten by my tag. */
-  alignas (sizeof (jitter_int) * 2)
+     overwritten by my tag.
+     This header is included from files using Gnulib macros, which appear to
+     redefine alignas in a way different from just _Alignas ; in order to avoid
+     these problems directly use _Alignas. */
+  _Alignas (sizeof (jitter_int) * 2)
 
   /* The primitive Lisp name as a C string. */
   char *name;
