@@ -1,6 +1,6 @@
 /* ios.c - IO spaces for poke.  */
 
-/* Copyright (C) 2019, 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -90,20 +90,31 @@ static struct ios *cur_io;
 /* The available backends are implemented in their own files, and
    provide the following interfaces.  */
 
+extern struct ios_dev_if ios_dev_zero; /* ios-dev-zero.c */
 extern struct ios_dev_if ios_dev_mem; /* ios-dev-mem.c */
 extern struct ios_dev_if ios_dev_file; /* ios-dev-file.c */
 extern struct ios_dev_if ios_dev_stream; /* ios-dev-stream.c */
 #ifdef HAVE_LIBNBD
 extern struct ios_dev_if ios_dev_nbd; /* ios-dev-nbd.c */
 #endif
+#ifdef HAVE_PROC
+extern struct ios_dev_if ios_dev_proc; /* ios-dev-proc.c */
+#endif
+extern struct ios_dev_if ios_dev_sub; /* ios-dev-sub.c */
 
 static struct ios_dev_if *ios_dev_ifs[] =
   {
+   NULL, /* Optional foreign IOD.  */
+   &ios_dev_zero,
    &ios_dev_mem,
    &ios_dev_stream,
 #ifdef HAVE_LIBNBD
    &ios_dev_nbd,
 #endif
+#ifdef HAVE_PROC
+   &ios_dev_proc,
+#endif
+   &ios_dev_sub,
    /* File must be last */
    &ios_dev_file,
    NULL,
@@ -136,20 +147,30 @@ ios_open (const char *handler, uint64_t flags, int set_cur)
     return IOS_ENOMEM;
 
   io->handler = NULL;
-  io->dev = NULL;
   io->next = NULL;
   io->bias = 0;
 
   /* Look for a device interface suitable to operate on the given
      handler.  */
-  for (dev_if = ios_dev_ifs; *dev_if; ++dev_if)
+  dev_if = ios_dev_ifs;
+  do
     {
-      iod_error = (*dev_if)->handler_normalize (handler, flags, &io->handler);
-      if (iod_error)
+      if (*dev_if == NULL)
+        {
+          /* Skip the foreign IO device if it is not set.  */
+          ++dev_if;
+          continue;
+        }
+
+      io->handler = (*dev_if)->handler_normalize (handler, flags, &iod_error);
+      if (iod_error != IOD_OK)
         goto error;
       if (io->handler)
         break;
+
+      ++dev_if;
     }
+  while (*dev_if);
 
   if (*dev_if == NULL)
     goto error;
@@ -165,7 +186,7 @@ ios_open (const char *handler, uint64_t flags, int set_cur)
       }
 
   /* Open the device using the interface found above.  */
-  iod_error = io->dev_if->open (handler, flags, &io->dev);
+  io->dev = io->dev_if->open (handler, flags, &iod_error, io->dev_if->data);
   if (iod_error || io->dev == NULL)
     goto error;
 
@@ -193,22 +214,20 @@ ios_open (const char *handler, uint64_t flags, int set_cur)
   return error;
 }
 
-void
+int
 ios_close (ios io)
 {
   struct ios *tmp;
-  int r;
+  int ret;
 
   /* XXX: if not saved, ask before closing.  */
 
   /* Close the device operated by the IO space.
-     XXX: handle errors.  */
-  r = io->dev_if->close (io->dev);
-  assert (r);
+     XXX: Errors may be received from fclose.  What do we do in that case?  */
+  ret = io->dev_if->close (io->dev);
 
   /* Unlink the IOS from the list.  */
-  assert (io_list != NULL); /* The list must contain at least one IO
-                               space.  */
+  assert (io_list != NULL); /* The list contains at least this IO space.  */
   if (io_list == io)
     io_list = io_list->next;
   else
@@ -223,6 +242,8 @@ ios_close (ios io)
     cur_io = io_list;
 
   free (io);
+
+  return IOD_ERROR_TO_IOS_ERROR (ret);
 }
 
 uint64_t
@@ -279,7 +300,7 @@ ios_get_id (ios io)
   return io->id;
 }
 
-char *
+const char *
 ios_get_dev_if_name (ios io)
 {
   return io->dev_if->get_if_name ();
@@ -319,9 +340,14 @@ void
 ios_map (ios_map_fn cb, void *data)
 {
   ios io;
+  ios io_next;
 
-  for (io = io_list; io; io = io->next)
-    (*cb) (io, data);
+  for (io = io_list; io; io = io_next)
+    {
+      /* Note that the handler may close IO.  */
+      io_next = io->next;
+      (*cb) (io, data);
+    }
 }
 
 /* Set all except the lowest SIGNIFICANT_BITS of VALUE to zero.  */
@@ -1486,6 +1512,10 @@ ios_write_int (ios io, ios_off offset, int flags,
                enum ios_nenc nenc,
                int64_t value)
 {
+  /* The IOS should be writable.  */
+  if (!(io->dev_if->get_flags (io->dev) & IOS_F_WRITE))
+    return IOS_EPERM;
+
   /* Apply the IOS bias.  */
   offset += ios_get_bias (io);
 
@@ -1507,6 +1537,10 @@ ios_write_uint (ios io, ios_off offset, int flags,
                 enum ios_endian endian,
                 uint64_t value)
 {
+  /* The IOS should be writable.  */
+  if (!(io->dev_if->get_flags (io->dev) & IOS_F_WRITE))
+    return IOS_EPERM;
+
   /* Apply the IOS bias.  */
   offset += ios_get_bias (io);
 
@@ -1524,6 +1558,10 @@ ios_write_string (ios io, ios_off offset, int flags,
 {
   const char *p;
 
+  /* The IOS should be writable.  */
+  if (!(io->dev_if->get_flags (io->dev) & IOS_F_WRITE))
+    return IOS_EPERM;
+
   /* Apply the IOS bias.  */
   offset += ios_get_bias (io);
 
@@ -1569,7 +1607,7 @@ ios_write_string (ios io, ios_off offset, int flags,
 uint64_t
 ios_size (ios io)
 {
-  return io->dev_if->size (io->dev) * 8;
+  return io->dev_if->size (io->dev);
 }
 
 int
@@ -1577,3 +1615,31 @@ ios_flush (ios io, ios_off offset)
 {
   return io->dev_if->flush (io->dev, offset / 8);
 }
+
+void *
+ios_get_dev (ios ios)
+{
+  return ios->dev;
+}
+
+struct ios_dev_if *
+ios_get_dev_if (ios ios)
+{
+  return ios->dev_if;
+}
+
+struct ios_dev_if *
+ios_foreign_iod (void)
+{
+  return ios_dev_ifs[0];
+}
+
+int
+ios_register_foreign_iod (struct ios_dev_if *iod_if)
+{
+  if (ios_dev_ifs[0] != NULL)
+    return IOS_ERROR;
+
+  ios_dev_ifs[0] = iod_if;
+  return IOS_OK;
+}
