@@ -1,6 +1,6 @@
 /* libpoke.c - Implementation of the public services in libpoke.  */
 
-/* Copyright (C) 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2020, 2021, 2022 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "pvm.h"
 #include "pvm-val.h" /* XXX */
 #include "libpoke.h"
+#include "ios-dev.h" /* for struct ios_dev_if */
 
 struct _pk_compiler
 {
@@ -44,7 +45,7 @@ struct pk_term_if libpoke_term_if;
 #define PK_RETURN(code) do { return pkc->status = (code); } while (0)
 
 pk_compiler
-pk_compiler_new (struct pk_term_if *term_if)
+pk_compiler_new_with_flags (struct pk_term_if *term_if, uint32_t flags)
 {
   pk_compiler pkc;
 
@@ -59,6 +60,11 @@ pk_compiler_new (struct pk_term_if *term_if)
   pkc = malloc (sizeof (struct _pk_compiler));
   if (pkc)
     {
+      uint32_t pkl_flags = 0;
+
+      if (flags & PK_F_NOSTDTYPES)
+        pkl_flags |= PKL_F_NOSTDTYPES;
+
       /* Determine the path to the compiler's runtime files.  */
       const char *libpoke_datadir = getenv ("POKEDATADIR");
       if (libpoke_datadir == NULL)
@@ -70,7 +76,8 @@ pk_compiler_new (struct pk_term_if *term_if)
       if (pkc->vm == NULL)
         goto error;
       pkc->compiler = pkl_new (pkc->vm,
-                               libpoke_datadir);
+                               libpoke_datadir,
+                               pkl_flags);
       if (pkc->compiler == NULL)
         goto error;
       pkc->complete_type = NULL;
@@ -86,6 +93,13 @@ pk_compiler_new (struct pk_term_if *term_if)
   return NULL;
 }
 
+pk_compiler
+pk_compiler_new (struct pk_term_if *term_if)
+{
+  return pk_compiler_new_with_flags (term_if, 0 /* flags */);
+}
+
+
 void
 pk_compiler_free (pk_compiler pkc)
 {
@@ -108,28 +122,30 @@ pk_errno (pk_compiler pkc)
 
 int
 pk_compile_file (pk_compiler pkc, const char *filename,
-                 int *exit_status)
+                 pk_val *exit_exception)
 {
-  PK_RETURN (pkl_execute_file (pkc->compiler, filename, exit_status)
-                 ? PK_OK
-                 : PK_ERROR);
+  PK_RETURN (pkl_execute_file (pkc->compiler, filename, exit_exception)
+             ? PK_OK
+             : PK_ERROR);
 }
 
 int
 pk_compile_buffer (pk_compiler pkc, const char *buffer,
-                   const char **end)
+                   const char **end, pk_val *exit_exception)
 {
-  PK_RETURN (pkl_execute_buffer (pkc->compiler, buffer, end) ? PK_OK
-                                                             : PK_ERROR);
+  PK_RETURN (pkl_execute_buffer (pkc->compiler, buffer,
+                                 end, exit_exception) ? PK_OK : PK_ERROR);
 }
 
 int
 pk_compile_statement (pk_compiler pkc, const char *buffer,
-                      const char **end, pk_val *valp)
+                      const char **end, pk_val *valp,
+                      pk_val *exit_exception)
 {
   pvm_val val;
 
-  if (!pkl_execute_statement (pkc->compiler, buffer, end, &val))
+  if (!pkl_execute_statement (pkc->compiler, buffer, end, &val,
+                              exit_exception))
     PK_RETURN (PK_ERROR);
 
   if (valp)
@@ -140,11 +156,12 @@ pk_compile_statement (pk_compiler pkc, const char *buffer,
 
 int
 pk_compile_expression (pk_compiler pkc, const char *buffer,
-                       const char **end, pk_val *valp)
+                       const char **end, pk_val *valp, pk_val *exit_exception)
 {
   pvm_val val;
 
-  if (!pkl_execute_expression (pkc->compiler, buffer, end, &val))
+  if (!pkl_execute_expression (pkc->compiler, buffer, end, &val,
+                               exit_exception))
     PK_RETURN (PK_ERROR);
 
   if (valp)
@@ -176,7 +193,7 @@ pk_set_lexical_cuckolding_p (pk_compiler pkc, int lexical_cuckolding_p)
 void
 pk_set_alien_token_fn (pk_compiler pkc, pk_alien_token_handler_fn cb)
 {
-  pkl_set_alien_token_fn (pkc->compiler, cb);
+  pkl_set_alien_token_fn (pkc->compiler, (pkl_alien_token_handler_fn) cb);
   pkc->status = PK_OK;
 }
 
@@ -512,7 +529,7 @@ pk_ios_get_id (pk_ios io)
   return ios_get_id ((ios) io);
 }
 
-char *
+const char *
 pk_ios_get_dev_if_name (pk_ios io)
 {
   return ios_get_dev_if_name ((ios) io);
@@ -524,6 +541,12 @@ pk_ios_size (pk_ios io)
   return ios_size ((ios) io);
 }
 
+uint64_t
+pk_ios_get_bias (pk_ios io)
+{
+  return ios_get_bias ((ios) io);
+}
+
 struct ios_map_fn_payload
 {
   pk_ios_map_fn cb;
@@ -550,6 +573,7 @@ pk_ios_map (pk_compiler pkc,
 struct decl_map_fn_payload
 {
   pk_map_decl_fn cb;
+  pvm_env runtime_env;
   void *data;
 };
 
@@ -562,6 +586,7 @@ my_decl_map_fn (pkl_ast_node decl, void *data)
   pkl_ast_node initial = PKL_AST_DECL_INITIAL (decl);
   pkl_ast_loc loc = PKL_AST_LOC (decl);
   char *source =  PKL_AST_DECL_SOURCE (decl);
+  int order = PKL_AST_DECL_ORDER (decl);
   char *type = NULL;
   int kind;
 
@@ -592,6 +617,7 @@ my_decl_map_fn (pkl_ast_node decl, void *data)
                type,
                loc.first_line, loc.last_line,
                loc.first_column, loc.last_column,
+               pvm_env_lookup (payload->runtime_env, 0, order),
                payload->data);
   free (type);
 }
@@ -600,7 +626,8 @@ void
 pk_decl_map (pk_compiler pkc, int kind,
              pk_map_decl_fn handler, void *data)
 {
-  struct decl_map_fn_payload payload = { handler, data };
+  struct decl_map_fn_payload payload
+      = { handler, pvm_get_env (pkc->vm), data };
   pkl_env compiler_env = pkl_get_env (pkc->compiler);
   int pkl_kind;
 
@@ -701,22 +728,22 @@ pk_defvar (pk_compiler pkc, const char *varname, pk_val val)
 }
 
 int
-pk_call (pk_compiler pkc, pk_val cls, pk_val *ret, ...)
+pk_call (pk_compiler pkc, pk_val cls, pk_val *ret, int narg, ...)
 {
   pvm_program program;
   va_list ap;
   enum pvm_exit_code rret;
 
   /* Compile a program that calls the function.  */
-  va_start (ap, ret);
-  program = pkl_compile_call (pkc->compiler, cls, ret, ap);
+  va_start (ap, narg);
+  program = pkl_compile_call (pkc->compiler, cls, ret, narg, ap);
   va_end (ap);
   if (!program)
     PK_RETURN (PK_ERROR);
 
   /* Run the program in the poke VM.  */
   pvm_program_make_executable (program);
-  rret = pvm_run (pkc->vm, program, ret);
+  rret = pvm_run (pkc->vm, program, ret, NULL);
 
   pvm_destroy_program (program);
   PK_RETURN (rret == PVM_EXIT_OK ? PK_OK : PK_ERROR);
@@ -916,9 +943,9 @@ pk_set_pretty_print (pk_compiler pkc, int pretty_print_p)
 }
 
 void
-pk_print_val (pk_compiler pkc, pk_val val)
+pk_print_val (pk_compiler pkc, pk_val val, pk_val *exit_exception)
 {
-  pvm_print_val (pkc->vm, val);
+  pvm_print_val (pkc->vm, val, exit_exception);
   pkc->status = PK_OK;
 }
 
@@ -926,9 +953,35 @@ void
 pk_print_val_with_params (pk_compiler pkc, pk_val val,
                           int depth, int mode, int base,
                           int indent, int acutoff,
-                          uint32_t flags)
+                          uint32_t flags,
+                          pk_val *exit_exception)
 {
   pvm_print_val_with_params (pkc->vm, val,
                              depth, mode, base,
-                             indent, acutoff, flags);
+                             indent, acutoff, flags,
+                             exit_exception);
+}
+
+static struct ios_dev_if foreign_iod_if;
+
+int
+pk_register_iod (pk_compiler pkc, struct pk_iod_if *iod_if)
+{
+  pkc->status = PK_OK;
+
+#define CF(FN) foreign_iod_if.FN = iod_if->FN
+  CF (get_if_name);
+  CF (handler_normalize);
+  CF (open);
+  CF (close);
+  CF (pread);
+  CF (pwrite);
+  CF (get_flags);
+  CF (size);
+  CF (flush);
+  CF (data);
+#undef CF
+
+  (void) ios_register_foreign_iod (&foreign_iod_if);
+  return pkc->status;
 }
