@@ -1,6 +1,6 @@
 /* pkl-ast.h - Abstract Syntax Tree for Poke.  */
 
-/* Copyright (C) 2019, 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,6 +57,8 @@ enum pkl_ast_code
   PKL_AST_FUNCALL_ARG,
   PKL_AST_VAR,
   PKL_AST_LAMBDA,
+  PKL_AST_FORMAT,
+  PKL_AST_FORMAT_ARG,
   PKL_AST_INCRDECR,
   PKL_AST_GCD,
   PKL_AST_LAST_EXP = PKL_AST_GCD,
@@ -88,7 +90,6 @@ enum pkl_ast_code
   PKL_AST_CONTINUE_STMT,
   PKL_AST_RAISE_STMT,
   PKL_AST_LAST_STMT = PKL_AST_RAISE_STMT,
-  PKL_AST_PRINT_STMT_ARG,
   PKL_AST_LAST
 };
 
@@ -403,6 +404,13 @@ pkl_ast_node pkl_ast_make_array_initializer (pkl_ast ast,
                                              pkl_ast_node index,
                                              pkl_ast_node exp);
 
+/* Concatenates two chain of array initializers and fixes the indices of
+   second initializer accordingly.  */
+
+pkl_ast_node
+pkl_ast_array_initializers_concat (pkl_ast ast,
+                                   pkl_ast_node init1,
+                                   pkl_ast_node init2);
 
 /* PKL_AST_STRUCT nodes represent struct literals.  */
 
@@ -752,6 +760,8 @@ pkl_ast_node pkl_ast_make_struct_ref (pkl_ast ast,
 
    TYPE is a PKL_AST_TYPE node.
 
+   SIZE is the size of TYPE if it is a complete type, or NULL.
+
    CONSTRAINT is a constraint associated with the struct field.  It is
    an expression that should evaluate to a boolean.
 
@@ -772,6 +782,7 @@ pkl_ast_node pkl_ast_make_struct_ref (pkl_ast ast,
 
 #define PKL_AST_STRUCT_TYPE_FIELD_NAME(AST) ((AST)->sct_type_elem.name)
 #define PKL_AST_STRUCT_TYPE_FIELD_TYPE(AST) ((AST)->sct_type_elem.type)
+#define PKL_AST_STRUCT_TYPE_FIELD_SIZE(AST) ((AST)->sct_type_elem.size)
 #define PKL_AST_STRUCT_TYPE_FIELD_CONSTRAINT(AST) ((AST)->sct_type_elem.constraint)
 #define PKL_AST_STRUCT_TYPE_FIELD_LABEL(AST) ((AST)->sct_type_elem.label)
 #define PKL_AST_STRUCT_TYPE_FIELD_ENDIAN(AST) ((AST)->sct_type_elem.endian)
@@ -784,6 +795,7 @@ struct pkl_ast_struct_type_field
 
   union pkl_ast_node *name;
   union pkl_ast_node *type;
+  union pkl_ast_node *size;
   union pkl_ast_node *constraint;
   union pkl_ast_node *initializer;
   union pkl_ast_node *label;
@@ -850,9 +862,9 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
    number of elements, then BOUND is an expression that must evaluate
    to an integer.  If the array type is bounded by size, then BOUND is
    an expression that must evaluate to an offset.  If the array type
-   is unbounded, then BOUND is NULL.  MAPPER, WRITER, PRINTER,
-   CONSTRUCTOR and BOUNDCLS are used to hold closures, or
-   PVM_NULL. The field LEX_CORRECTED is used by the transl phase, to
+   is unbounded, then BOUND is NULL.  MAPPER, WRITER, FORMATER,
+   PRINTER, CONSTRUCTOR, INTEGRATOR and BOUNDCLS are used to hold closures,
+   or PVM_NULL. The field LEX_CORRECTED is used by the transl phase, to
    keep record of array types that have been lexically corrected; this
    is to avoid processing the same type more than once.
 
@@ -861,10 +873,10 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
    declarations.  ELEMS is a chain of elements, which can be
    PKL_AST_STRUCT_TYPE_FIELD or PKL_AST_DECL nodes, potentially mixed.
    PINNED_P is 1 if the struct is pinned, 0 otherwise.  MAPPER, WRITER
-   CONSTRUCTOR, PRINTER, COMPARATOR and INTEGRATOR are used to hold
-   closures, or PVM_NULL.  INT_TYPE, if not NULL, is an AST node with
-   an integral type, that defines the nature of this struct type as
-   integral.
+   CONSTRUCTOR, FORMATER, PRINTER, COMPARATOR, INTEGRATOR and
+   DEINTEGRATOR are used to hold closures, or PVM_NULL.  ITYPE, if
+   not NULL, is an AST node with an integral type, that defines the
+   nature of this struct type as integral.
 
    In offset types, BASE_TYPE is a PKL_AST_TYPE with the base type for
    the offset's magnitude, and UNIT is either a PKL_AST_IDENTIFIER
@@ -898,6 +910,8 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
 #define PKL_AST_TYPE_A_BOUNDER(AST) ((AST)->type.val.array.closures[2])
 #define PKL_AST_TYPE_A_CONSTRUCTOR(AST) ((AST)->type.val.array.closures[3])
 #define PKL_AST_TYPE_A_PRINTER(AST) ((AST)->type.val.array.closures[4])
+#define PKL_AST_TYPE_A_FORMATER(AST) ((AST)->type.val.array.closures[5])
+#define PKL_AST_TYPE_A_INTEGRATOR(AST) ((AST)->type.val.array.closures[6])
 #define PKL_AST_TYPE_S_NFIELD(AST) ((AST)->type.val.sct.nfield)
 #define PKL_AST_TYPE_S_NDECL(AST) ((AST)->type.val.sct.ndecl)
 #define PKL_AST_TYPE_S_NELEM(AST) ((AST)->type.val.sct.nelem)
@@ -911,6 +925,9 @@ pkl_ast_node pkl_ast_make_func_type_arg (pkl_ast ast,
 #define PKL_AST_TYPE_S_PRINTER(AST) ((AST)->type.val.sct.closures[3])
 #define PKL_AST_TYPE_S_COMPARATOR(AST) ((AST)->type.val.sct.closures[4])
 #define PKL_AST_TYPE_S_INTEGRATOR(AST) ((AST)->type.val.sct.closures[5])
+#define PKL_AST_TYPE_S_FORMATER(AST) ((AST)->type.val.sct.closures[6])
+#define PKL_AST_TYPE_S_DEINTEGRATOR(AST) ((AST)->type.val.sct.closures[7])
+#define PKL_AST_TYPE_S_TYPIFIER(AST) ((AST)->type.val.sct.closures[8])
 #define PKL_AST_TYPE_S_ITYPE(AST) ((AST)->type.val.sct.itype)
 #define PKL_AST_TYPE_O_UNIT(AST) ((AST)->type.val.off.unit)
 #define PKL_AST_TYPE_O_BASE_TYPE(AST) ((AST)->type.val.off.base_type)
@@ -945,8 +962,8 @@ struct pkl_ast_type
     {
       union pkl_ast_node *bound;
       union pkl_ast_node *etype;
-      /* Uncollectable array for MAPPER, BOUNDER, WRITER, CONSTRUCTOR
-         and PRINTER.  */
+      /* Uncollectable array for MAPPER, BOUNDER, WRITER, CONSTRUCTOR,
+         INTEGRATOR, PRINTER and FORMATER.  */
       pvm_val *closures;
     } array;
 
@@ -959,8 +976,9 @@ struct pkl_ast_type
       union pkl_ast_node *itype;
       int pinned_p;
       int union_p;
-      /* Uncollectable array for MAPPER, WRITER, CONSTRUCTOR, COMPARATOR,
-         INTEGRATOR and PRINTER.  */
+      /* Uncollectable array for MAPPER, WRITER, CONSTRUCTOR,
+         COMPARATOR, INTEGRATOR, DEINTEGRATOR, PRINTER, FORMATER, and
+         TYPIFIER.  */
       pvm_val *closures;
     } sct;
 
@@ -1017,8 +1035,12 @@ int pkl_ast_type_is_exception (pkl_ast_node type);
 int pkl_ast_type_promoteable_p (pkl_ast_node ft, pkl_ast_node tt,
                                 int promote_array_of_any);
 
+int pkl_ast_type_integrable_p (pkl_ast_node type);
+
 pkl_ast_node pkl_ast_sizeof_type (pkl_ast ast, pkl_ast_node type);
 
+size_t pkl_ast_sizeof_integral_type (pkl_ast_node type);
+
 int pkl_ast_type_is_complete (pkl_ast_node type);
 
 void pkl_ast_array_type_remove_bounders (pkl_ast_node type);
@@ -1029,6 +1051,10 @@ char *pkl_type_str (pkl_ast_node type, int use_given_name);
 
 pkl_ast_node pkl_struct_type_traverse (pkl_ast_node type, const char *path);
 
+pkl_ast_node pkl_type_integral_promote (pkl_ast ast,
+                                        pkl_ast_node type1,
+                                        pkl_ast_node type2);
+
 /* Return an expression that evaluates to an increment step for the
    given TYPE.  If the provided type doesn't support the notion of
    increment step this function returns NULL.  */
@@ -1287,6 +1313,100 @@ struct pkl_ast_funcall_arg
 pkl_ast_node pkl_ast_make_funcall_arg (pkl_ast ast, pkl_ast_node exp,
                                        pkl_ast_node name);
 
+/* PKL_AST_FORMAT nodes represent `format'.
+
+   ARGS is a chained list of PKL_AST_FORMAT_ARG nodes.  This may
+   contain zero or more nodes, and the types of their expressions
+   should match the format expressed in FMT.
+
+   FMT is the format string node.
+
+   TYPES is a linked list of type nodes, corresponding to the %-
+   directives in FMT.  Note that %<class> and %</class> directives
+   have type PKL_TYPE_VOID.
+
+   NARGS is the number of arguments in ARGS.
+
+   PREFIX, if not NULL, is a C string that comes before the arguments.
+
+   FMT_PROCESSED_P is a boolean indicating whether the format string
+   has been already processed in this node.  */
+
+#define PKL_AST_FORMAT_FMT(AST) ((AST)->format.fmt)
+#define PKL_AST_FORMAT_TYPES(AST) ((AST)->format.types)
+#define PKL_AST_FORMAT_ARGS(AST) ((AST)->format.args)
+#define PKL_AST_FORMAT_NARGS(AST) ((AST)->format.nargs)
+#define PKL_AST_FORMAT_PREFIX(AST) ((AST)->format.prefix)
+#define PKL_AST_FORMAT_FMT_PROCESSED_P(AST) ((AST)->format.fmt_processed_p)
+
+struct pkl_ast_format
+{
+  struct pkl_ast_common common;
+
+  int nargs;
+  char *prefix;
+  int fmt_processed_p;
+  union pkl_ast_node *fmt;
+  union pkl_ast_node *types;
+  union pkl_ast_node *args;
+};
+
+pkl_ast_node pkl_ast_make_format (pkl_ast ast, pkl_ast_node fmt,
+                                  pkl_ast_node args, int printf_p);
+
+/* PKL_AST_FORMAT_ARG nodes represent expression arguments to `format'.
+
+   EXP is an expression node evaluating to the value to format.
+
+   BASE is the numeration base to use when formating this argument.
+
+   BEGIN_SC, if not NULL, marks that this argument is a %<class>
+   directive, and is a NULL-terminated string with the name of the
+   styling class to begin.
+
+   END_SC, if not NULL, marks that this argument is a %</class>
+   directive, and is a NULL-terminated string with the name of the
+   styling class to end.
+
+   SUFFIX, if not NULL, is a C string that should be appended after the
+   value of EXP, respectively.
+
+   VALUE_P indicates whether the argument shall be printed as a PVM
+   value or not (whether this arguement corresponds to a %v or not).
+
+   FORMAT_MODE and FORMAT_DEPTH specify how the argument shall be
+   printed if VALUE_P is true.  FORMAT_MODE can be one of the
+   PKL_AST_FORMAT_MODE_* constants defined below, while FORMAT_DEPTH can
+   be zero or a positive integer.  */
+
+#define PKL_AST_FORMAT_ARG_EXP(AST) ((AST)->format_arg.exp)
+#define PKL_AST_FORMAT_ARG_BASE(AST) ((AST)->format_arg.base)
+#define PKL_AST_FORMAT_ARG_SUFFIX(AST) ((AST)->format_arg.suffix)
+#define PKL_AST_FORMAT_ARG_BEGIN_SC(AST) ((AST)->format_arg.begin_sc)
+#define PKL_AST_FORMAT_ARG_END_SC(AST) ((AST)->format_arg.end_sc)
+#define PKL_AST_FORMAT_ARG_VALUE_P(AST) ((AST)->format_arg.value_p)
+#define PKL_AST_FORMAT_ARG_FORMAT_MODE(AST) ((AST)->format_arg.format_mode)
+#define PKL_AST_FORMAT_ARG_FORMAT_DEPTH(AST) ((AST)->format_arg.format_depth)
+
+#define PKL_AST_FORMAT_MODE_FLAT 0
+#define PKL_AST_FORMAT_MODE_TREE 1
+
+struct pkl_ast_format_arg
+{
+  struct pkl_ast_common common;
+
+  char *begin_sc;
+  char *end_sc;
+  int base;
+  int value_p;
+  int format_mode;
+  int format_depth;
+  char *suffix;
+  union pkl_ast_node *exp;
+};
+
+pkl_ast_node pkl_ast_make_format_arg (pkl_ast ast, pkl_ast_node exp);
+
 /* PKL_AST_VAR nodes represent variable references.
 
    NAME is a PKL_AST_IDENTIFIER containing the name used to refer to
@@ -1431,7 +1551,24 @@ pkl_ast_node pkl_ast_make_incrdecr (pkl_ast ast,
 #define PKL_AST_BUILTIN_TERM_BEGIN_HYPERLINK 21
 #define PKL_AST_BUILTIN_TERM_END_HYPERLINK 22
 #define PKL_AST_BUILTIN_IOFLAGS 23
-#define PKL_AST_BUILTIN_UNSAFE_STRING_SET 24
+#define PKL_AST_BUILTIN_SLEEP 24
+#define PKL_AST_BUILTIN_IOGETB 25
+#define PKL_AST_BUILTIN_IOSETB 26
+#define PKL_AST_BUILTIN_VM_OBASE 27
+#define PKL_AST_BUILTIN_VM_SET_OBASE 28
+#define PKL_AST_BUILTIN_VM_OPPRINT 29
+#define PKL_AST_BUILTIN_VM_SET_OPPRINT 30
+#define PKL_AST_BUILTIN_VM_OACUTOFF 31
+#define PKL_AST_BUILTIN_VM_SET_OACUTOFF 32
+#define PKL_AST_BUILTIN_VM_ODEPTH 33
+#define PKL_AST_BUILTIN_VM_SET_ODEPTH 34
+#define PKL_AST_BUILTIN_VM_OINDENT 35
+#define PKL_AST_BUILTIN_VM_SET_OINDENT 36
+#define PKL_AST_BUILTIN_VM_OMAPS 37
+#define PKL_AST_BUILTIN_VM_SET_OMAPS 38
+#define PKL_AST_BUILTIN_VM_OMODE 39
+#define PKL_AST_BUILTIN_VM_SET_OMODE 40
+#define PKL_AST_BUILTIN_UNSAFE_STRING_SET 41
 
 struct pkl_ast_comp_stmt
 {
@@ -1593,12 +1730,16 @@ pkl_ast_node pkl_ast_make_loop_stmt_iterator (pkl_ast ast,
    NDROPS is the number of stack elements to drop before returning
    from the function.
 
+   NPOPES is the number of exception stack elements to drop before
+   returning from the function.
+
    FUNCTION is the PKL_AST_FUNCTION containing this return
    statement. */
 
 #define PKL_AST_RETURN_STMT_EXP(AST) ((AST)->return_stmt.exp)
 #define PKL_AST_RETURN_STMT_NFRAMES(AST) ((AST)->return_stmt.nframes)
 #define PKL_AST_RETURN_STMT_NDROPS(AST) ((AST)->return_stmt.ndrops)
+#define PKL_AST_RETURN_STMT_NPOPES(AST) ((AST)->return_stmt.npopes)
 #define PKL_AST_RETURN_STMT_FUNCTION(AST) ((AST)->return_stmt.function)
 
 struct pkl_ast_return_stmt
@@ -1609,6 +1750,7 @@ struct pkl_ast_return_stmt
   union pkl_ast_node *function;
   int nframes;
   int ndrops;
+  int npopes;
 };
 
 pkl_ast_node pkl_ast_make_return_stmt (pkl_ast ast, pkl_ast_node exp);
@@ -1687,103 +1829,28 @@ pkl_ast_node pkl_ast_make_try_catch_stmt (pkl_ast ast,
                                           pkl_ast_node code, pkl_ast_node handler,
                                           pkl_ast_node arg, pkl_ast_node exp);
 
-/* PKL_AST_PRINT_STMT nodes represent `print' statements.
+/* PKL_AST_PRINT_STMT nodes represent `print' and `printf' statements.
 
-   ARGS is a chained list of PKL_AST_PRINT_STMT_ARG nodes.  In `print'
-   statements, this only contains one argument, whose expression
-   should be of type string.  In `printf' statements, this may contain
-   zero or more nodes, and the types of their expressions should match
-   teh format expressed in FMT.
+   STR_EXP, if not NULL, is an expression node of type string.  In
+   `print' statements this is not NULL, but in `printf' statements
+   this should be NULL.
 
-   FMT, if not NULL, is a format string node.
+   FORMAT, if not NULL, is a PKL_AST_FORMAT node capturing all the
+   information corresponding to the `printf' statements.  */
 
-   TYPES is a linked list of type nodes, corresponding to the %-
-   directives in FMT.  Note that %<class> and %</class> directives
-   have type PKL_TYPE_VOID.
-
-   NARGS is the number of arguments in ARGS.
-
-   PREFIX, if not NULL, is a C string that should be printed before
-   the arguments.
-
-   FMT_PROCESSED_P is a boolean indicating whether the format string
-   has been already processed in this node.  */
-
-#define PKL_AST_PRINT_STMT_FMT(AST) ((AST)->print_stmt.fmt)
-#define PKL_AST_PRINT_STMT_TYPES(AST) ((AST)->print_stmt.types)
-#define PKL_AST_PRINT_STMT_ARGS(AST) ((AST)->print_stmt.args)
-#define PKL_AST_PRINT_STMT_NARGS(AST) ((AST)->print_stmt.nargs)
-#define PKL_AST_PRINT_STMT_PREFIX(AST) ((AST)->print_stmt.prefix)
-#define PKL_AST_PRINT_STMT_FMT_PROCESSED_P(AST) ((AST)->print_stmt.fmt_processed_p)
+#define PKL_AST_PRINT_STMT_STR_EXP(AST) ((AST)->print_stmt.str_exp)
+#define PKL_AST_PRINT_STMT_FORMAT(AST) ((AST)->print_stmt.format)
 
 struct pkl_ast_print_stmt
 {
   struct pkl_ast_common common;
 
-  int nargs;
-  char *prefix;
-  int fmt_processed_p;
-  union pkl_ast_node *fmt;
-  union pkl_ast_node *types;
-  union pkl_ast_node *args;
+  union pkl_ast_node *str_exp;
+  union pkl_ast_node *format;
 };
 
 pkl_ast_node pkl_ast_make_print_stmt (pkl_ast ast,
-                                      pkl_ast_node fmt, pkl_ast_node args);
-
-/* PKL_AST_PRINT_STMT_ARG nodes represent expression arguments to
-   `printf' statements.
-
-   EXP is an expression node evaluating to the value to print.
-
-   BASE is the numeration base to use when printing this argument.
-
-   BEGIN_SC, if not NULL, marks that this argument is a %<class>
-   directive, and is a NULL-terminated string with the name of the
-   styling class to begin.
-
-   END_SC, if not NULL, marks that this argument is a %</class>
-   directive, and is a NULL-terminated string with the name of the
-   styling class to end.
-
-   SUFFIX, if not NULL, is a C string that should be printed after the
-   value of EXP, respectively.
-
-   VALUE_P indicates whether the argument shall be printed as a PVM
-   value or not.
-
-   PRINT_MODE and PRINT_DEPTH specify how the argument shall be
-   printed if VALUE_P is true.  PRINT_MODE can be one of the
-   PKL_AST_PRINT_MODE_* constants defined below, while PRINT_DEPTH can
-   be zero or a positive integer.  */
-
-#define PKL_AST_PRINT_STMT_ARG_EXP(AST) ((AST)->print_stmt_arg.exp)
-#define PKL_AST_PRINT_STMT_ARG_BASE(AST) ((AST)->print_stmt_arg.base)
-#define PKL_AST_PRINT_STMT_ARG_SUFFIX(AST) ((AST)->print_stmt_arg.suffix)
-#define PKL_AST_PRINT_STMT_ARG_BEGIN_SC(AST) ((AST)->print_stmt_arg.begin_sc)
-#define PKL_AST_PRINT_STMT_ARG_END_SC(AST) ((AST)->print_stmt_arg.end_sc)
-#define PKL_AST_PRINT_STMT_ARG_VALUE_P(AST) ((AST)->print_stmt_arg.value_p)
-#define PKL_AST_PRINT_STMT_ARG_PRINT_MODE(AST) ((AST)->print_stmt_arg.print_mode)
-#define PKL_AST_PRINT_STMT_ARG_PRINT_DEPTH(AST) ((AST)->print_stmt_arg.print_depth)
-
-#define PKL_AST_PRINT_MODE_FLAT 0
-#define PKL_AST_PRINT_MODE_TREE 1
-
-struct pkl_ast_print_stmt_arg
-{
-  struct pkl_ast_common common;
-
-  char *begin_sc;
-  char *end_sc;
-  int base;
-  int value_p;
-  int print_mode;
-  int print_depth;
-  char *suffix;
-  union pkl_ast_node *exp;
-};
-
-pkl_ast_node pkl_ast_make_print_stmt_arg (pkl_ast ast, pkl_ast_node exp);
+                                      int printf_p, pkl_ast_node fmt);
 
 /* PKL_AST_CONTINUE_STMT nodes represent `continue' statements.  Each
    continue statement is associated to a loop node.
@@ -1874,6 +1941,8 @@ union pkl_ast_node
   struct pkl_ast_funcall_arg funcall_arg;
   struct pkl_ast_var var;
   struct pkl_ast_lambda lambda;
+  struct pkl_ast_format format;
+  struct pkl_ast_format_arg format_arg;
   struct pkl_ast_incrdecr incrdecr;
   /* Types.  */
   struct pkl_ast_type type;
@@ -1900,7 +1969,6 @@ union pkl_ast_node
   struct pkl_ast_continue_stmt continue_stmt;
   struct pkl_ast_raise_stmt raise_stmt;
   struct pkl_ast_print_stmt print_stmt;
-  struct pkl_ast_print_stmt_arg print_stmt_arg;
 };
 
 static inline pkl_ast_node __attribute__ ((always_inline, warn_unused_result))
